﻿using System;
using System.Runtime.InteropServices;

namespace Oni
{
    [StructLayout(LayoutKind.Sequential)]
    internal struct Vector2 : IEquatable<Vector2>
	{
		public float X;
		public float Y;

		public Vector2(float x, float y)
		{
			X = x;
			Y = y;
		}

        public Vector2(float all)
        {
            X = all;
            Y = all;
        }

        public static Vector2 FromArray(float[] values, int index)
		{
			Vector2 v;

			v.X = values[index * 2 + 0];
			v.Y = values[index * 2 + 1];

			return v;
		}

        public void CopyTo(float[] values, int index)
        {
            values[index + 0] = X;
            values[index + 1] = Y;
        }

		public static Vector2 operator +(Vector2 v1, Vector2 v2)
		{
			v1.X += v2.X;
			v1.Y += v2.Y;

			return v1;
		}

		public static Vector2 operator -(Vector2 v1, Vector2 v2)
		{
			v1.X -= v2.X;
			v1.Y -= v2.Y;

			return v1;
		}

		public static bool operator ==(Vector2 v1, Vector2 v2)
		{
			return (v1.X == v2.X && v1.Y == v2.Y);
		}

		public static bool operator !=(Vector2 v1, Vector2 v2)
		{
			return (v1.X != v2.X || v1.Y != v2.Y);
		}

		public static float Dot(Vector2 v1, Vector2 v2)
		{
			return v1.X * v2.X + v1.Y * v2.Y;
		}

        public static Vector2 Normalize(Vector2 v)
        {
            return v * (1.0f / v.Length());
        }

        public void Normalize()
		{
			float f = 1.0f / Length();

			X *= f;
			Y *= f;
		}

		public float Length()
		{
			return FMath.Sqrt(X * X + Y * Y);
		}

        public static Vector2 operator *(Vector2 v, float s)
        {
            v.X *= s;
            v.Y *= s;

            return v;
        }

        public static Vector2 operator *(float s, Vector2 v)
        {
            v.X *= s;
            v.Y *= s;

            return v;
        }

        public static Vector2 operator /(Vector2 v, float s)
        {
            return v * (1.0f / s);
        }

        public static Vector2 Min(Vector2 v1, Vector2 v2)
        {
            v1.X = (v1.X < v2.X) ? v1.X : v2.X;
            v1.Y = (v1.Y < v2.Y) ? v1.Y : v2.Y;

            return v1;
        }

        public static Vector2 Max(Vector2 v1, Vector2 v2)
        {
            v1.X = (v1.X > v2.X) ? v1.X : v2.X;
            v1.Y = (v1.Y > v2.Y) ? v1.Y : v2.Y;

            return v1;
        }

        public bool Equals(Vector2 other)
		{
			return (X == other.X && Y == other.Y);
		}

		public override bool Equals(object obj)
		{
			if (obj is Vector2)
				return Equals((Vector2)obj);

			return false;
		}

		public override int GetHashCode()
		{
			return X.GetHashCode() ^ Y.GetHashCode();
		}

		public override string ToString()
		{
			return String.Format("{{{0} {1}}}", X, Y);
		}

		private static Vector2 zero = new Vector2(0.0f, 0.0f);
		private static Vector2 one = new Vector2(1.0f, 1.0f);
		private static Vector2 unitX = new Vector2(1.0f, 0.0f);
		private static Vector2 unitY = new Vector2(0.0f, 1.0f);

		public static Vector2 Zero
		{
			get
			{
				return zero;
			}
		}

		public static Vector2 One
		{
			get
			{
				return one;
			}
		}

		public static Vector2 UnitX
		{
			get
			{
				return unitX;
			}
		}

		public static Vector2 UnitY
		{
			get
			{
				return unitY;
			}
		}
	}
}
